<?php

namespace Drupal\Tests\system_user\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\null_user\NullUser;
use Drupal\system_user\Service\SystemUser;
use Drupal\user\Entity\User;
use Drupal\user\UserInterface;

class SystemUserTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'system_user',
    'user',
  ];

  /**
   * The system user service.
   *
   * @var \Drupal\system_user\Service\SystemUser
   */
  private $systemUser;

  /**
   * An array of test users.
   *
   * @var \Drupal\user\UserInterface[]
   */
  private $users = [];

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();

    $this->installSchema('system', ['sequences']);

    $this->installEntitySchema('user');

    $this->systemUser = $this->container->get('system_user.system_user');
  }

  /**
   * Test that all system users can be retrieved.
   */
  public function testAllUsersCanBeRetrieved() {
    $this->assertEmpty($this->systemUser->getAll());

    $this->createUsers();

    $this->assertCount(3, $this->systemUser->getAll());
  }

  /**
   * Test that a single system user can be retrieved.
   */
  public function testFirstUserCanBeRetrieved() {
    $this->assertInstanceOf(NullUser::class, $this->systemUser->getFirst());

    $this->createUsers();

    $this->assertInstanceOf(UserInterface::class, $this->systemUser->getFirst());

    $this->assertEquals(2, $this->systemUser->getFirst()->id());
  }

  /**
   * Test that a user can be identified as a system user.
   */
  public function testSystemUserCheck() {
    $this->createUsers();

    $this->assertFalse(SystemUser::isSystemUser($this->users[0]));

    $this->assertTrue(SystemUser::isSystemUser($this->users[1]));
  }

  /**
   * Create some test users.
   */
  private function createUsers() {
    $this->users[] = $this->createUser([SystemUser::FIELD_NAME => FALSE]);
    $this->users[] = $this->createUser([SystemUser::FIELD_NAME => TRUE]);
    $this->users[] = $this->createUser([SystemUser::FIELD_NAME => FALSE]);
    $this->users[] = $this->createUser([SystemUser::FIELD_NAME => TRUE]);
    $this->users[] = $this->createUser([SystemUser::FIELD_NAME => FALSE]);
    $this->users[] = $this->createUser([SystemUser::FIELD_NAME => TRUE]);
  }

  /**
   * Create a single test user.
   *
   * @param array $overrides
   *   An array of values to add to or override the defaults.
   *
   * @return \Drupal\Core\Entity\EntityInterface
   *   The created user.
   */
  private function createUser(array $overrides = []) {
    $values = [
      'name' => $this->randomString(),
    ];

    $user = User::create(array_merge($values, $overrides));
    $user->save();

    return $user;
  }

}
